# RTLS API Gateway

## Table of Contents

<!-- START doctoc generated TOC please keep comment here to allow auto update -->
<!-- DON'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->

- [Features](#features)
- [Tech Stack](#tech-stack)
- [Architecture Overview](#architecture-overview)
- [Development env configuration](#development-env-configuration)
    - [Prerequisites](#prerequisites)
    - [Clone the project](#clone-the-project)
- [Configuration](#configuration)
- [Usage](#usage)
    - [Running the Service Locally](#running-the-service-locally)
    - [API Endpoints](#api-endpoints)
- [API Documentation](#api-documentation)
- [Running Tests](#running-tests)
- [Package Release](#package-release)

<!-- END doctoc generated TOC please keep comment here to allow auto update -->

## Features

This project is designed as an HTTP proxy router that listens on a single port and directs incoming HTTP requests to one
of the predefined destination services, based on specific routing rules. It simplifies backend management by
centralizing traffic handling through one entry point, distributing requests efficiently to multiple services.

## Tech Stack

- **Language**: JavaScript
- **Platform**: Node.js
- **Framework**: Express-gateway

## Architecture Overview

The Express-Gateway runs locally on port `8082` and serves as the core component for routing and proxying HTTP requests.

This GitHub repository contains a web server, named the `RTLS API Gateway`, which implements gateway logic to route HTTP
requests from clients to various backend services, each responsible for specific functions within the system:

- **Operational REST API Server** (port `8083`) - Manages core entities and handles operational data requests.
- **Historical REST API Server** (port `3000`) - Provides access to historical data and query capabilities.
- **Platform REST API Server** (port `3001`) - Oversees the management of primary entities within the platform.
- **Neighbor List Collector REST API Server** (port `3302`) - Contains mechanisms for neighbor list collection,
  formation, and updates.
- **System Health Check REST API Server** (port `8087`) - Generates health check reports and allows retrieval of the
  latest system status.

## Development env configuration

### Prerequisites

- Ubuntu 22.04 (Jammy Jellyfish)
- [install node v20 with yarn](https://github.com/rpplabs/rtls-frontend/blob/dev/README.md#install-nodejs)

### Clone the project

- Add public ssh key to [your account](https://github.com/settings/keys)

- Clone the repository

```shell
# via https
git clone https://github.com/rpplabs/rtls-api-gateway.git

# OR via ssh key
git clone git@github.com:rpplabs/rtls-api-gateway.git

cd rtls-api-gateway

```

## Configuration

- install server node packages

```shell
npm install

```

```shell
cp .env.defaults .env

```

- Update values on `.env` file with new values if required

## Usage

### Running the Service Locally

```shell
npm start

```

- The service will be available at: `http://localhost:8082`

### API Endpoints

## API Documentation

* [gateway.config.yml](config/gateway.config.yml)

## Running Tests

```shell
npm test

```

## Package Release

- Reset all current changes, checkout and update current development branch(`dev`)

```shell
git checkout -- ./
git checkout dev
git pull origin dev

```

- Run scripts for version increment

```shell
yarn version:patch

```

- Create new build branch (`master`)

```shell
git branch -D master
git branch master

```

- Make **force push**(for the case if `master` branch already exists from branch differ than `dev`)

```shell
git push origin master -f 

```

- Check build progress in [jenkins](https://jenkins.rpplabs.com/job/rtls-web-team/job/rtls-api-gateway/job/master/)
- After build complete package will locate in [repository](https://repo.rpplabs.com/ubuntu/) as `rtls-api-gateway*`
