const chai = require('chai');
const sinon  = require('sinon');
const httpMocks = require('node-mocks-http');
const MemoryStore = require('../concurrentRateLimit/policies/MemoryStore');
const rateLimit = require('../concurrentRateLimit/policies/concurrentRateLimit');


const expect = chai.expect;


describe('Concurrent Rate Limit Policy', () => {
    describe('MemoryStore', () => {
        it('should return instance of MemoryStore class', () => {
            const store = new MemoryStore();
            expect(store).to.be.an.instanceof(MemoryStore);
        });
        
        it('should provide increment functionality', () => {
            const store = new MemoryStore();
            expect(store.concurrent).to.equal(0);
            store.increment();
            expect(store.concurrent).to.equal(1);
            store.increment();
            expect(store.concurrent).to.equal(2);
        });
    
        it('should provide decrement functionality', () => {
            const store = new MemoryStore();
            expect(store.concurrent).to.equal(0);
            store.increment();
            expect(store.concurrent).to.equal(1);
            store.decrement();
            expect(store.concurrent).to.equal(0);
            store.decrement();
            expect(store.concurrent).to.equal(0);
        });
    });
    
    describe('concurrentRateLimit', () => {
        it('should provide "concurrent-rate-limit" policy name', () => {
            expect(rateLimit.name).to.equal('concurrent-rate-limit');
        });
    
        it('should provide "concurrent-rate-limit" policy name', () => {
            const request  = httpMocks.createRequest({
                method: 'GET',
                url: '/user/42',
                params: {
                    id: 42
                }
            });
            request.egContext = { evaluateAsTemplateString: () => { return 'test'; } };
            const response = httpMocks.createResponse();
            const nextSpy = sinon.spy();
            const rateMiddleware = rateLimit.policy({});
            
            rateMiddleware(request, response, nextSpy);
            
            expect(nextSpy.calledOnce).to.equal(true);
        })
    })
});
