const MemoryStore = require('./MemoryStore');


const store = new Map();
store.set('totalLimit', new MemoryStore());

module.exports = {
    name: 'concurrent-rate-limit',
    policy: (actionParams) => {
        return (req, res, next) => {
            actionParams.keyGenerator = (req) => {
                try {
                    return req.egContext.evaluateAsTemplateString(actionParams.rateLimitBy);
                } catch (err) {
                    console.error('Failed to generate rate-limit key with config: %s; %s', actionParams.rateLimitBy, err.message);
                }
            };
            
            const limitedBy = actionParams.keyGenerator(req);
            if (!store.get(limitedBy)) {
                store.set(limitedBy, new MemoryStore());
            }
            
            const limitedByStore = store.get(limitedBy);
            const totalLimitStore = store.get('totalLimit');
            const totalLimitNotExceeded = totalLimitStore.concurrent < actionParams.totalConcurrent;
            const limitByNotExceeded = limitedByStore.concurrent < actionParams.maxConcurrent;

            if (totalLimitNotExceeded) {
                totalLimitStore.increment();
                if (limitByNotExceeded) {
                    limitedByStore.increment();
                } else {
                    res.status(429).send('Too many requests, please try again later.')
                }
            } else {
                res.status(429).send('Too many requests, please try again later.')
            }

            res.on('finish', function(){
                if (totalLimitNotExceeded) {
                    totalLimitStore.decrement();
                }
                if (limitByNotExceeded) {
                    limitedByStore.decrement();
                }
            });
    
            res.on('close', function(){
                totalLimitStore.decrement();
                limitedByStore.decrement();
            });
    
            next();
        };
    }
};
