#!/usr/bin/env bash
set -x #echo on

npm install

PROJECT=rtls-api-gateway
# WORKSPACE="/home/rtls/rtls-api-gateway/"

RPM_DESCRIPTION="Redpoint Positioning RTLS Api Gateway"
RPM_MAINTAINER=help@redpointpositioning.com
RPP_VENDOR="Redpoint Positioning, Inc."

PROJECT_HOME=${WORKSPACE}

npm install

export PATH=/usr/local/bin:/usr/local/rbenv/shims:$PATH

VERSION="$( sys-services/rtls-get-version.sh )" # 2021.3.36
RELEASE="$( sys-services/rtls-get-release.sh )" # 0
FILE="$( sys-services/rtls-get-file-name.sh )" # rtls-api-gateway_2021.3.36-0_amd64.deb

echo "PROJECT_HOME:       ${PROJECT_HOME}"
echo "BUILDING ON BRANCH: ${BRANCH_NAME}"
echo "VERSION:            ${VERSION}"
echo "RELEASE:            ${RELEASE}"
echo "FILE:               ${FILE}"

rm -f "${PROJECT_HOME}"/*.deb
rm -f "${PROJECT_HOME}"/*.rpm
[ -d "${PROJECT_HOME}"/releases ] && rm -f "${PROJECT_HOME}"/releases/*

fpm -s dir -t deb -f \
	--description "${RPM_DESCRIPTION}" \
	--maintainer  "${RPM_MAINTAINER}" \
	--vendor "${RPP_VENDOR}" \
	--license "Proprietary" \
	--provides "rtls-api-gateway" \
	--exclude ".git" \
	--prefix /usr/rtls/rtls-api-gateway \
	--rpm-use-file-permissions --rpm-user rtls --rpm-group rtls \
	--before-remove sys-services/pre-uninstall.sh \
	--before-install sys-services/pre-install.sh \
	--after-install sys-services/post-install.sh \
	--before-upgrade sys-services/pre-update.sh \
	--after-upgrade sys-services/post-update.sh \
	--post-uninstall sys-services/post-uninstall.sh \
	--depends "nodejs (>= 20.5.1)" \
	--deb-group rtls \
  --deb-user rtls \
  --name "${PROJECT}" \
	--version "${VERSION}" \
	--iteration "${RELEASE}" \
	--package "${PROJECT_HOME}" \
	.
